Attribute VB_Name = "File"
'---------------------------------------------------------
' PROGRAMMING VB6 ROUTINE LIBRARY - (C) Francesco Balena
'
' A collection of file routines
'---------------------------------------------------------

Option Explicit

' returns the attributes of a file in a readable format
' this routine also works with open files
' raises an error if the file doesn't exist

Function GetAttrDescr(filename As String) As String
    Dim result As String, attr As Long
    attr = GetAttr(filename)
    ' GetAttr also works with directories
    If attr And vbDirectory Then result = result & " Directory"
    If attr And vbReadOnly Then result = result & " ReadOnly"
    If attr And vbHidden Then result = result & " Hidden"
    If attr And vbSystem Then result = result & " System"
    If attr And vbArchive Then result = result & " Archive"
    ' discard the first space
    GetAttrDescr = Mid$(result, 2)
End Function

' return an array of strings holding all the filenames that
' match a given filespec and search attributes

Function GetFiles(filespec As String, Optional Attributes As _
    VbFileAttribute) As String()
        Dim result() As String
        Dim filename As String, count As Long, path2 As String
        Const ALLOC_CHUNK = 50
        ReDim result(0 To ALLOC_CHUNK) As String
        filename = Dir$(filespec, Attributes)
        Do While Len(filename)
            count = count + 1
            If count > UBound(result) Then
                ' resize the result array, if necessary
                ReDim Preserve result(0 To count + ALLOC_CHUNK) As String
            End If
            result(count) = filename
            ' get ready for the next iteration
            filename = Dir$
        Loop
        ' trim the result array
        ReDim Preserve result(0 To count) As String
        GetFiles = result
End Function

' check if a file exists

Function FileExists(filename As String) As Boolean
    On Error Resume Next
    FileExists = (Dir$(filename) <> "")
End Function

' check if a directory exists

Function DirExists(path As String) As Boolean
    On Error Resume Next
    DirExists = (Dir$(path & "\nul") <> "")
End Function

' return an array of strings holding all the subdirectories in a path
' that match search attributes - optionally it returns the entire path

Function GetDirectories(path As String, Optional Attributes As _
    VbFileAttribute, Optional IncludePath As Boolean) As String()
        Dim result() As String
        Dim dirname As String, count As Long, path2 As String
        Const ALLOC_CHUNK = 50
        ReDim result(ALLOC_CHUNK) As String
        ' build the path name + backslash
        path2 = path
        If Right$(path2, 1) <> "\" Then path2 = path2 & "\"
        dirname = Dir$(path2 & "*.*", vbDirectory Or Attributes)
        Do While Len(dirname)
            If dirname = "." Or dirname = ".." Then
                ' exclude the "." and ".." entries
            ElseIf (GetAttr(path2 & dirname) And vbDirectory) = 0 Then
                ' this is a regular file
            Else
                ' this is a directory
                count = count + 1
                If count > UBound(result) Then
                    ' resize the result array, if necessary
                    ReDim Preserve result(count + ALLOC_CHUNK) As String
                End If
                ' include the path if requested
                If IncludePath Then dirname = path2 & dirname
                result(count) = dirname
            End If
            dirname = Dir$
        Loop
        ' trim the result array
        ReDim Preserve result(count) As String
        GetDirectories = result
End Function

' load the names of all executable files in a directory tree into a ListBox
' Note: this is a recursive routine

Sub ListExecutableFiles(ByVal path As String, lst As ListBox)
    Dim names() As String, i As Long, j As Integer
    ' ensure that there is a trailing backslash
    If Right(path, 1) <> "\" Then path = path & "\"
    ' get the list of executable files
    For j = 1 To 4
        ' at each iteration search for a different extension
        names() = GetFiles(path & "*." & _
            Choose(j, "exe", "bat", "com", "pif"))
        ' load partial results in the listbox
        For i = 1 To UBound(names)
            lst.AddItem path & names(i)
        Next
    Next
    ' get the list of subdirectories, including hidden ones
    ' and call this routine recursively on all of them
    names() = GetDirectories(path, vbHidden)
    For i = 1 To UBound(names)
        ListExecutableFiles path & names(i), lst
    Next
End Sub

' load a text file into a TextBox control

Sub LoadFileInTextBox(filename As String, txt As TextBox)
    Dim fnum As Integer, isOpen As Boolean
    Dim textline As String, Text As String
    On Error GoTo Error_Handler
    ' get the next free file number
    fnum = FreeFile()
    Open filename For Input As #fnum
    ' if execution flow got here, the files has been open without error
    isOpen = True
    Do Until EOF(fnum)
        Line Input #fnum, textline
        Text = Text & textline & vbCrLf
    Loop
    ' load into the textbox
    txt.Text = Text
    ' intentionally flow into the error handler to close the file
Error_Handler:
    ' raise the error (if any), but first close the file
    If isOpen Then Close #fnum
    If Err Then Err.Raise Err.number, , Err.Description
End Sub

' returns the entire contents of a text file as a string

Function ReadTextFileContents(filename As String) As String
    Dim fnum As Integer, isOpen As Boolean
    On Error GoTo Error_Handler
    ' get the next free file number
    fnum = FreeFile()
    Open filename For Input As #fnum
    ' if execution flow got here, the file has been open without error
    isOpen = True
    ' read the entire contents in one single operation
    ReadTextFileContents = Input(LOF(fnum), fnum)
    ' intentionally flow into the error handler to close the file
Error_Handler:
    ' raise the error (if any), but first close the file
    If isOpen Then Close #fnum
    If Err Then Err.Raise Err.number, , Err.Description
End Function

' writes the contents of a string to a file, optionally in Append mode

Sub WriteTextFileContents(Text As String, filename As String, _
    Optional AppendMode As Boolean)
        Dim fnum As Integer, isOpen As Boolean
        On Error GoTo Error_Handler
        ' get the next free file number
        fnum = FreeFile()
        If AppendMode Then
            Open filename For Append As #fnum
        Else
            Open filename For Output As #fnum
        End If
        ' if execution flow got here, the file has been open correctly
        isOpen = True
        ' print to the file in one single operation
        Print #fnum, Text
        ' intentionally flow into the error handler to close the file
Error_Handler:
        ' raise the error (if any), but first close the file
        If isOpen Then Close #fnum
        If Err Then Err.Raise Err.number, , Err.Description
End Sub

' returns the contents of a text file as an array of strings
' NOTE: requires the ReadTextFileContents routine

Function GetTextFileLines(filename As String, Optional DropEmpty As Boolean, _
    Optional Limit As Variant) As String()
        Dim FileText As String, items() As String, i As Long
        ' read the file's contents, exit if any error
        FileText = ReadTextFileContents(filename)
        ' this is necessary, because Split() only accepts 1-char delimiters
        FileText = Replace(FileText, vbCrLf, vbCr)
        ' split the file in individual lines of text
        items() = Split(FileText, vbCr, Limit)
        ' drop empty lines, if requested
        If DropEmpty Then
            ' fill empty lines with something that other items
            ' surely don't contain
            For i = 0 To UBound(items)
                If Len(items(i)) = 0 Then items(i) = vbCrLf
            Next
            ' use the Filter() function to quickly drop empty lines
            items() = Filter(items(), vbCrLf, False)
        End If
        GetTextFileLines = items()
End Function

' returns the contents of a delimited text file as an array of strings arrays
' NOTE: requires the ReadTextFileContents and GetTextFileLines routines

Function ImportDelimitedFile(filename As String, _
    Optional delimiter As String = vbTab) As Variant()
        Dim lines() As String, i As Long
        ' get all lines in the file, skipping over blank ones
        lines() = GetTextFileLines(filename, True)
        ' create a string array out of each line of text
        ' and store it into a Variant element
        ReDim values(0 To UBound(lines)) As Variant
        For i = 0 To UBound(lines)
            values(i) = Split(lines(i), delimiter, -1)
        Next
        ImportDelimitedFile = values()
End Function

' write the contents of an array of string arrays to a delimited text file
' NOTE: requires the WriteTextFileContents routine

Sub ExportDelimitedFile(values() As Variant, filename As String, _
    Optional delimiter As String = vbTab)
        Dim i As Long, j As Long, TextFile As String
        ' rebuild the individual lines of text of the file
        ReDim lines(0 To UBound(values)) As String
        For i = 0 To UBound(values)
            lines(i) = Join(values(i), delimiter)
        Next
        ' create CRLFs among records
        TextFile = Replace(Join(lines, vbCr), vbCr, vbCrLf)
        WriteTextFileContents TextFile, filename
End Sub

' Duplicate a directory tree without also copying the files

' call this routine to initiate the copy process
' NOTE: the destination folder is created if necessary
'       uses the Private Sub DuplicateDirTreeSub procedure

Sub DuplicateDirTree(SourcePath As String, DestPath As String)
    Dim fso As New Scripting.FileSystemObject
    Dim sourceFld As Scripting.Folder, destFld As Scripting.Folder
    ' the source folder must exists
    Set sourceFld = fso.GetFolder(SourcePath)
    ' the destination folder is created if necessary
    If fso.FolderExists(DestPath) Then
        Set destFld = fso.GetFolder(DestPath)
    Else
        Set destFld = fso.CreateFolder(DestPath)
    End If
    ' jump to the recursive routine to do the real job
    DuplicateDirTreeSub sourceFld, destFld
End Sub

' Private recursive procedure used by DuplicateDirTree

Private Sub DuplicateDirTreeSub(source As Folder, destination As Folder)
    Dim sourceFld As Scripting.Folder, destFld As Scripting.Folder
    For Each sourceFld In source.SubFolders
        ' copy this subfolder into destination folder
        Set destFld = destination.SubFolders.Add(sourceFld.Name)
        ' then repeat the process recursively for all
        ' the subfolders of the folder just considered
        DuplicateDirTreeSub sourceFld, destFld
    Next
End Sub

' search all the TXT files in a directory for a string

' for each matching file returns a Variant element that contains a
' 3-item array that holds the filename, the line and the column line
' NOTE: all searches are case insensitive

Function SearchTextFiles(path As String, search As String) As Variant()
    Dim fso As New Scripting.FileSystemObject
    Dim fil As Scripting.File, ts As Scripting.TextStream
    Dim pos As Long, count As Long
    ReDim result(50) As Variant
    ' search for all the TXT files in the directory
    For Each fil In fso.GetFolder(path).Files
        If UCase$(fso.GetExtensionName(fil.path)) = "TXT" Then
            ' get the corresponding TextStream object
            Set ts = fil.OpenAsTextStream(ForReading)
            ' read its contents, search the string, close it
            pos = InStr(1, ts.ReadAll, search, vbTextCompare)
            ts.Close
            If pos > 0 Then
                ' if the string has been found, reopen the file
                ' to determine its position in terms of (line,column)
                Set ts = fil.OpenAsTextStream(ForReading)
                ' skip all preceding characters to get where we want
                ts.Skip pos - 1
                ' fill the result array, make room if necessary
                count = count + 1
                If count > UBound(result) Then
                    ReDim Preserve result(UBound(result) + 50) As Variant
                End If
                ' each result item is a three-element array
                result(count) = Array(fil.path, ts.Line, ts.Column)
                ' now we may close the TextStrem
                ts.Close
            End If
        End If
    Next
    ' resize the result array to indicate number of matches
    ReDim Preserve result(0 To count) As Variant
    SearchTextFiles = result
End Function



