Attribute VB_Name = "ListBoxAPI"
'---------------------------------------------------
' Reusable routines for ListBox and ComboBox controls.
'---------------------------------------------------

Option Explicit

Private Declare Function SendMessage Lib "user32" Alias "SendMessageA" (ByVal hwnd As Long, ByVal wMsg As Long, ByVal wParam As Long, lParam As Any) As Long
Private Declare Function SendMessageByVal Lib "user32" Alias "SendMessageA" (ByVal hwnd As Long, ByVal wMsg As Long, ByVal wParam As Long, ByVal lParam As Long) As Long
Private Declare Function SendMessageString Lib "user32" Alias "SendMessageA" (ByVal hwnd As Long, ByVal wMsg As Long, ByVal wParam As Long, ByVal lParam As String) As Long
Private Declare Sub CopyMemory Lib "kernel32" Alias "RtlMoveMemory" (dest As Any, dest As Any, ByVal numBytes As Long)
Private Declare Function GetClientRect Lib "user32" (ByVal hwnd As Long, lpRect As RECT) As Long

Type RECT
    Left As Long
    Top As Long
    Right As Long
    Bottom As Long
End Type

Const LB_FINDSTRING = &H18F
Const LB_FINDSTRINGEXACT = &H1A2
Const LB_GETHORIZONTALEXTENT = &H193
Const LB_GETITEMHEIGHT = &H1A1
Const LB_GETITEMRECT = &H198
Const LB_GETSELCOUNT = &H190
Const LB_GETSELITEMS = &H191
Const LB_SELITEMRANGE = &H19B
Const LB_SETHORIZONTALEXTENT = &H194
Const LB_SETITEMHEIGHT = &H1A0
Const LB_SETTABSTOPS = &H192
Const LB_ITEMFROMPOINT = &H1A9

Const CB_FINDSTRING = &H14C
Const CB_FINDSTRINGEXACT = &H158
Const CB_GETDROPPEDCONTROLRECT = &H152
Const CB_GETDROPPEDSTATE = &H157
Const CB_GETITEMHEIGHT = &H154
Const CB_SETITEMHEIGHT = &H153
Const CB_SHOWDROPDOWN = &H14F
Const CB_GETHORIZONTALEXTENT = &H15D
Const CB_SETHORIZONTALEXTENT = &H15E
Const CB_GETDROPPEDWIDTH = &H15F
Const CB_SETDROPPEDWIDTH = &H160
Const CB_LIMITTEXT = &H141

' Find a string in the control.
' The third argument is the index *after* which to start the search (first item if omitted).
' If the fourth argument is True it searches for an exact match.
' Returns the index of the match, or -1 if not found.

Function FindString(ctrl As Control, ByVal search As String, Optional startIndex As Long = -1, Optional ExactMatch As Boolean) As Long
    Dim uMsg As Long
    If TypeOf ctrl Is ListBox Then
        uMsg = IIf(ExactMatch, LB_FINDSTRINGEXACT, LB_FINDSTRING)
    ElseIf TypeOf ctrl Is ComboBox Then
        uMsg = IIf(ExactMatch, CB_FINDSTRINGEXACT, CB_FINDSTRING)
    Else
        Exit Function
    End If
    FindString = SendMessageString(ctrl.hwnd, uMsg, startIndex, search)
End Function

' Return the height of each entry in the control (in pixels).

Function GetItemHeight(ctrl As Control) As Long
    Dim uMsg As Long
    If TypeOf ctrl Is ListBox Then
        uMsg = LB_GETITEMHEIGHT
    ElseIf TypeOf ctrl Is ComboBox Then
        uMsg = CB_GETITEMHEIGHT
    Else
        Exit Function
    End If
    GetItemHeight = SendMessageByVal(ctrl.hwnd, uMsg, 0, 0)
End Function

' Set the height in pixels of each entry in the control.

Sub SetItemHeight(ctrl As Control, ByVal newHeight As Long)
    Dim uMsg As Long
    If TypeOf ctrl Is ListBox Then
        uMsg = LB_SETITEMHEIGHT
    ElseIf TypeOf ctrl Is ComboBox Then
        uMsg = CB_SETITEMHEIGHT
    Else
        Exit Sub
    End If
    ' (only the low-order word of lParam can be used.)
    SendMessageByVal ctrl.hwnd, uMsg, 0, (newHeight And &HFFFF&)
    ' It is necessary to manually refresh the control.
    ctrl.Refresh
End Sub

'--------------------------------------------------
' ListBox specific routines
'--------------------------------------------------

' Return the horizontal extent of the control (in pixel).

Function GetHorizontalExtent(lb As ListBox) As Long
    GetHorizontalExtent = SendMessageByVal(lb.hwnd, LB_GETHORIZONTALEXTENT, 0, 0)
End Function

' Set the horizontal extent of the control (in pixel).
' If this value is greater than the current control's width
' a horizontal scrollbar appears.

Sub SetHorizontalExtent(lb As ListBox, ByVal newWidth As Long)
    SendMessageByVal lb.hwnd, LB_SETHORIZONTALEXTENT, newWidth, 0
End Sub

' Select or deselect all the items in a range.

Sub SelItemRange(lb As ListBox, ByVal newState As Boolean, Optional firstItem As Long, Optional lastItem As Long = -1)
    Dim lParam As Long
    ' account for omitted lastItem argument.
    If lastItem < 0 Then lastItem = lb.ListCount - 1
    ' lParam must contain the first item's index in its low-order word
    ' and the last item's index in its high-order word.
    ' We need this method to avoid overflow.
    CopyMemory lParam, firstItem, 2
    CopyMemory ByVal VarPtr(lParam) + 2, lastItem, 2
    SendMessageByVal lb.hwnd, LB_SELITEMRANGE, newState, lParam
End Sub

' Set tab stops. Each element of the array is expressed in dialog units,
' where each dialog unit is 1/4 of the average character width.

Sub SetTabStops(lb As ListBox, tabStops() As Long)
    Dim numEls As Long
    numEls = UBound(tabStops) - LBound(tabStops) + 1
    SendMessage lb.hwnd, LB_SETTABSTOPS, numEls, tabStops(LBound(tabStops))
End Sub

' Return an array with the indexes of all the selected items.

Function GetSelItems(lb As ListBox) As Long()
    Dim selCount As Long
    ' Retrieve the number of selected items (same as SelCount property)
    selCount = SendMessageByVal(lb.hwnd, LB_GETSELCOUNT, 0, 0)
    ' If no item is selected, return an unitialized array
    If selCount = 0 Then Exit Function
    
    ReDim result(1 To selCount) As Long
    SendMessage lb.hwnd, LB_GETSELITEMS, selCount, result(1)
    GetSelItems = result()
End Function

' Return the index of the item at given coordinates.

Function ItemFromPoint(lb As ListBox, ByVal X As Long, ByVal Y As Long) As Long
    Dim lParam As Long
    ' lParam must contain X in its low-order word
    ' and Y in its high-order word.
    ' We need this method to avoid overflow.
    CopyMemory lParam, X, 2
    CopyMemory ByVal VarPtr(lParam) + 2, Y, 2
    ItemFromPoint = SendMessageByVal(lb.hwnd, LB_ITEMFROMPOINT, 0, lParam)
End Function

' Return the number of visible items.

Function VisibleItems(lb As ListBox) As Long
    Dim lpRect As RECT, itemHeight As Long
    ' Get client rectangle area.
    GetClientRect lb.hwnd, lpRect
    ' Get height of each item.
    itemHeight = SendMessageByVal(lb.hwnd, LB_GETITEMHEIGHT, 0, 0)
    ' Do the division.
    VisibleItems = (lpRect.Bottom - lpRect.Top) \ itemHeight
End Function

'--------------------------------------------------
' ComboBox specific routines
'--------------------------------------------------

' Show or hide the drop-down list.

Sub ShowDropDown(cb As ComboBox, Optional ByVal showIt As Boolean = True)
    SendMessageByVal cb.hwnd, CB_SHOWDROPDOWN, showIt, 0
End Sub

' Return the dropped state.

Function GetDroppedState(cb As ComboBox) As Boolean
    GetDroppedState = SendMessageByVal(cb.hwnd, CB_GETDROPPEDSTATE, 0, 0)
End Function

' Return the dimension of the drop-down list rectangle.

Sub GetDroppedControlRect(cb As ComboBox, Left As Long, Top As Long, Right As Long, Bottom As Long)
    Dim lpRect As RECT
    SendMessage cb.hwnd, CB_GETDROPPEDCONTROLRECT, 0, lpRect
    Left = lpRect.Left
    Top = lpRect.Top
    Right = lpRect.Right
    Bottom = lpRect.Bottom
End Sub

' Return the minimum width of the drop-down list portion.

Function GetDroppedWidth(cb As ComboBox) As Long
    GetDroppedWidth = SendMessageByVal(cb.hwnd, CB_GETDROPPEDWIDTH, 0, 0)
End Function

' Set the minimum width of the drop-down list portion.

Sub SetDroppedWidth(cb As ComboBox, ByVal newWidth As Long)
    SendMessageByVal cb.hwnd, CB_SETDROPPEDWIDTH, newWidth, 0
End Sub

' Set the maximum number of characters in the control.
' (similar to the MaxLength property)

Sub SetMaxLength(cb As ComboBox, ByVal MaxLength As Long)
    SendMessageByVal cb.hwnd, CB_LIMITTEXT, MaxLength, 0
End Sub
