Attribute VB_Name = "Math"
'---------------------------------------------------------
' PROGRAMMING VB6 ROUTINE LIBRARY - (C) Francesco Balena
'
' A collection of math routines
'---------------------------------------------------------

Option Explicit

' a replacement for the "^" operator, when the exponent is an integer value
' it is at least 50% faster in native-compiled code

Function Power(ByVal number As Double, ByVal exponent As Long) As Double
    Dim result As Double
    If exponent < 0 Then Err.Raise 5  ' invalid procedure call or argument
    result = 1
    Do While exponent
        If exponent And 1 Then result = result * number
        number = number * number
        exponent = exponent \ 2
    Loop
    Power = result
End Function

' a MOD replacement that also works with floating point numbers
' Note: this differs from MOD when number is negative

Function FPMod(ByVal number As Double, ByVal divisor As Double) As Double
    FPMod = number - Int(number / divisor) * divisor
End Function

' base-10 logarithm

Function Log10(number As Double) As Double
    Log10 = Log(number) / 2.30258509299405
End Function

' "equal" up to their 10th decimal digit

Function AlmostEqual(x, y) As Boolean
    AlmostEqual = (Abs(x - y) <= 0.0000000001)
End Function

' a very fast routine that returns 2^N, with N in the range [0-31]

Function Power2(ByVal exponent As Long) As Long
    Static result(0 To 31) As Long, i As Integer
    ' evaluate powers of two only once
    If result(0) = 0 Then
        result(0) = 1
        For i = 1 To 30
            result(i) = result(i - 1) * 2
        Next
        result(31) = &H80000000        ' this is a special value
    End If
    Power2 = result(exponent)
End Function

' fractional part of a floating-point number
' note that negative numbers return negative values

Function Fract(number As Double) As Double
    Fract = number - Fix(number)
End Function

' the Greatest Common Divisor of two integers - Euclide's algorithm

Function Gcd(n1 As Long, n2 As Long) As Long
    Dim one As Long, two As Long, tmp As Long
    one = n1: two = n2
    If one > two Then
        tmp = one: one = two: two = tmp
    End If
    
    Do
        tmp = one: one = two: two = tmp
        one = one Mod two
    Loop While one
    Gcd& = two
End Function

' the Least Common Multiple of two integers - Euclide's algorithm

Function Lcm(n1 As Long, n2 As Long) As Long
    Dim one As Long, two As Long, tmp As Long
    one = n1: two = n2: tmp = n1 * n2
    If one > two Then
        tmp = one: one = two: two = tmp
    End If
                                                                              
    Do
        tmp = one: one = two: two = tmp
        one = one Mod two
    Loop While one
    Lcm = tmp \ two
End Function

' the factorial of a number (n = 0-170)

Function Factorial(number As Long) As Double
    Static result(170) As Double, i As Long
    ' illegal procedure call if negative
    If number < 0 Then Err.Raise 5
    ' overflow if too high
    If number > 170 Then Err.Raise 6
    
    ' evalutate all possible values in advance
    If result(0) = 0 Then
        result(0) = 1
        For i = 1 To UBound(result)
            result(i) = result(i - 1) * CDbl(i)
        Next
    End If
    Factorial = result(number)
End Function

' number of combinations of ITEMS elements in groups of GROUPSIZE
' combinations are all the possible arrangements you can combine N object
' in groups of K without distinguishing results that only differ in their order
' For example: Combinations(52,5) gives you the number of card hands you can receive

' Note: uses the Factorial routine

Function Combinations(items As Long, groupsize As Long) As Double
    Combinations = (Factorial(items) / Factorial(items - groupsize)) / Factorial(groupsize)
End Function

' number of permutations of ITEMS elements in groups of GROUPSIZE
' permutations are all the possible arrangements you can combine N object
' in groups of K when you distinguish results that differ in their order
' For example: Permutations(10,3) returns the number of permutation in which
' 1st, 2nd and 3rd prize can be assigned to a game with 10 playes

' Note: uses the Factorial routine

Function Permutations(items As Long, groupsize As Long) As Double
    Permutations = Factorial(items) / Factorial(items - groupsize)
End Function

' returns the nearest integer equal or higher than the argument

Function Ceiling(number As Double) As Long
    Ceiling = -Int(-number)
End Function

' return an array with the first N prime numbers

Function PrimeArray(numberOfPrimes As Long) As Long()
    Dim found As Long, n As Long, i As Long
    If numberOfPrimes <= 0 Then Err.Raise 1002, , "Invalid argument"
    ' we know the size of the result in advance
    ReDim result(1 To numberOfPrimes) As Long
    ' "2" is the first prime number
    result(1) = 2: found = 1
    n = 1
    Do While found < numberOfPrimes
        ' all other prime numbers are odd, so we can skip even numbers
        n = n + 2
        ' let's check if N is a prime number
        For i = 1 To found
            If (n Mod result(i)) = 0 Then Exit For
        Next
        If i > found Then
            ' no prime number < N is a divisor for N, therefore N is prime
            found = found + 1
            result(found) = n
        End If
    Loop
    PrimeArray = result
End Function

' a function that sums all the values you pass to it
' or concatenates them if they are strings

Function Sum(ParamArray args() As Variant) As Double
    Dim i As Integer, result As Double
    For i = LBound(args) To UBound(args)
        result = result + args(i)
    Next
    Sum = result
End Function

' a function that returns the highest value among passed arguments

Function Max(first As Double, ParamArray args() As Variant) As Double
    Dim i As Integer, result As Double
    result = first
    For i = LBound(args) To UBound(args)
        If args(i) > result Then result = args(i)
    Next
    Max = result
End Function

' a function that returns the lowest value among passed arguments

Function Min(first As Double, ParamArray args() As Variant) As Double
    Dim i As Integer, result As Double
    result = first
    For i = LBound(args) To UBound(args)
        If args(i) < result Then result = args(i)
    Next
    Min = result
End Function


