Attribute VB_Name = "Strings"
'---------------------------------------------------------
' PROGRAMMING VB6 ROUTINE LIBRARY - (C) Francesco Balena
'
' A collection of string routines
'---------------------------------------------------------

Option Explicit

' return True if a string holds a valid numeric value
' works better than IsDecimal, and can reject non-integer values

Function CheckNumeric(Text As String, DecValue As Boolean) As Boolean
    Dim i As Integer
    For i = 1 To Len(Text)
        Select Case Mid$(Text, i, 1)
            Case "0" To "9"
            Case "-", "+"
                ' minus/plus sign are only allowed
                ' as leading chars
                If i > 1 Then Exit Function
            Case "."
                ' exit if decimal values not allowed
                If Not DecValue Then Exit Function
                ' only one decimal separator is allowed
                If InStr(Text, ".") < i Then Exit Function
            Case Else
                ' reject all other characters
                Exit Function
        End Select
    Next
    CheckNumeric = True
End Function

' filter out all invalid characters in a string

Function FilterString(Text As String, validChars As String) As String
    Dim i As Long, result As String
    For i = 1 To Len(Text)
        If InStr(validChars, Mid$(Text, i, 1)) Then
            result = result & Mid$(Text, i, 1)
        End If
    Next
    FilterString = result
End Function

' filters out all formatting characters in a string that contains a number
' correctly accounts for the current decimal separator character

Function FilterNumber(Text As String, TrimZeroes As Boolean) As String
    Dim decSep As String, i As Long, result As String
    ' retrieve the decimal separator symbol
    decSep = Format$(0.1, ".")
    ' use FilterString for most of the work
    result = FilterString(Text, decSep & "-0123456789")
    ' only if there is a decimal part and the
    ' user requested to trim non-significant digits
    If TrimZeroes And InStr(Text, decSep) > 0 Then
        For i = Len(result) To 1 Step -1
            Select Case Mid$(result, i, 1)
                Case decSep
                    result = Left$(result, i - 1)
                    Exit For
                Case "0"
                    result = Left$(result, i - 1)
                Case Else
                    Exit For
            End Select
        Next
    End If
    FilterNumber = result
End Function

' format a phone-number in the format "XXX-XXX-XXXX"

Function FormatPhoneNumber(Text As String) As String
    Dim tmp As String
    If Text <> "" Then
        ' first, get rid of all embedded dashes, if any
        tmp = FilterString(Text, "0123456789")
        ' then, re-insert them in the correct position
        If Len(tmp) <= 7 Then
            FormatPhoneNumber = Format$(tmp, "!@@@-@@@@")
        Else
            FormatPhoneNumber = Format$(tmp, "!@@@-@@@-@@@@")
        End If
    End If
End Function

' format a credit card number, in the format "XXXX XXXX XXXX XXXX"

Function FormatCreditCard(Text As String) As String
    Dim tmp As String
    If Text <> "" Then
        ' first, get rid of all embedded spaces,if any
        tmp = FilterNumber(Text, False)
        ' then, re-insert them in the correct position
        FormatCreditCard = Format$(tmp, "!@@@@ @@@@ @@@@ @@@@")
    End If
End Function

' count the number of sub-strings

Function InstrCount(source As String, search As String, Optional start As Long = 1, _
    Optional Compare As VbCompareMethod = vbBinaryCompare) As Long
        Dim i As Long, result As Long
        i = InStr(start, source, search, Compare)
        Do While i
            result = result + 1
            i = InStr(i + 1, source, search, Compare)
        Loop
        InstrCount = result
End Function

' search the first occurrence of a character among those
' listed in SEARCHTABLE

Function InstrTbl(source As String, searchTable As String, Optional start As Long = 1, _
    Optional Compare As VbCompareMethod = vbBinaryCompare) As Long
        Dim i As Long
        For i = start To Len(source)
            If InStr(1, searchTable, Mid$(source, i, 1), Compare) Then
                InstrTbl = i
                Exit For
            End If
        Next
End Function

' search the last occurrence of a character among those
' listed in SEARCHTABLE

Function InstrTblRev(source As String, searchTable As String, Optional ByVal start As Long = -1, _
    Optional Compare As VbCompareMethod = vbBinaryCompare) As Long
        Dim i As Long
        If start < 0 Then start = Len(source)
        For i = start To 1 Step -1
            If InStr(1, searchTable, Mid$(source, i, 1), Compare) Then
                InstrTblRev = i
                Exit For
            End If
        Next
End Function

' a replacement for the built-in Split function, that works with
' multiple delimited chars

Function SplitMulti(source As String, delimTable As String, Optional Limit As Long = -1, _
    Optional Compare As VbCompareMethod = vbBinaryCompare) As String()
        Dim i As Long, start As Long, count As Long
        Dim result() As String
        ' we allocate the result array in chunks, to reduce overhead
        Const ALLOC_CHUNK = 50
        ReDim result(0 To ALLOC_CHUNK - 1) As String
        ' parse the string
        For i = 1 To Len(source)
            If InStr(1, delimTable, Mid$(source, i, 1), Compare) Then
                ' we have found a delimiter
                If start Then
                    ' if we were in the middle of a word
                    If count > UBound(result) Then
                        ' make room in the result array if necessary
                        ReDim Preserve result(0 To UBound(result) _
                            + ALLOC_CHUNK) As String
                    End If
                    result(count) = Mid$(source, start, i - start)
                    start = 0
                    count = count + 1
                    If count = Limit Then Exit For
                End If
            ElseIf start = 0 Then
                ' this isn't a delimiter - remember where the word starts
                start = i
            End If
        Next
        ' if we were in the middle of a word
        If start And count <> Limit Then
            ' make room in the result array if necessary, or trim it
            ReDim Preserve result(0 To count) As String
            result(count) = Mid$(source, start)
        Else
           ' trim the array to the actual number of items
           ReDim Preserve result(0 To count - 1) As String
        End If
        SplitMulti = result()
End Function

' optimized routine that counts the spaces in a string
' NOTE: this function doesn't work with non-Latin alphabets

Function CountSpaces(Text As String) As Long
    Dim b() As Byte, i As Long, result As Long
    b() = Text
    For i = 0 To UBound(b) Step 2
        ' only consider even-numbered items
        If b(i) = 32 Then result = result + 1
    Next
    CountSpaces = result
End Function


