Attribute VB_Name = "TextBoxAPI"
'---------------------------------------------------
' Reusable routines for TextBox controls.
'---------------------------------------------------

Option Explicit

Private Declare Function SendMessage Lib "user32" Alias "SendMessageA" (ByVal hwnd As Long, ByVal wMsg As Long, ByVal wParam As Long, lParam As Any) As Long
Private Declare Function SendMessageByVal Lib "user32" Alias "SendMessageA" (ByVal hwnd As Long, ByVal wMsg As Long, ByVal wParam As Long, ByVal lParam As Long) As Long

Type RECT
    Left As Long
    Top As Long
    Right As Long
    Bottom As Long
End Type

Const EM_CANUNDO = &HC6
Const EM_UNDO = &HC7
Const EM_EMPTYUNDOBUFFER = &HCD
Const EM_GETLINECOUNT = &HBA
Const EM_GETFIRSTVISIBLELINE = &HCE
Const EM_LINEFROMCHAR = &HC9
Const EM_LINEINDEX = &HBB
Const EM_LINELENGTH = &HC1
Const EM_FMTLINES = &HC8
Const EM_GETLINE = &HC4
Const EM_GETRECT = &HB2
Const EM_SCROLL = &HB5
Const EM_SCROLLCARET = &HB7
Const EM_SETRECT = &HB3
Const EM_SETTABSTOPS = &HCB
Const EM_LINESCROLL = &HB6

'-------------------------------------------
' Undo operations
'-------------------------------------------

' Return True if the most recent edit operation can be undone.
Function CanUndo(tb As TextBox) As Boolean
    CanUndo = SendMessageByVal(tb.hwnd, EM_CANUNDO, 0, 0)
End Function

' Undo the most recent edit operation.
Sub Undo(tb As TextBox)
    SendMessageByVal tb.hwnd, EM_UNDO, 0, 0
End Sub

' Empty the undo buffer.
Sub EmptyUndoBuffer(tb As TextBox)
    SendMessageByVal tb.hwnd, EM_EMPTYUNDOBUFFER, 0, 0
End Sub

'-------------------------------------------
' Multi-line controls
'-------------------------------------------

' Return the number of lines in the control.

Function GetLineCount(tb As TextBox) As Long
    GetLineCount = SendMessageByVal(tb.hwnd, EM_GETLINECOUNT, 0, 0)
End Function

' Return the index of the first visible line
' (0 for the first text line in the control).
' When applied to a single-line control, return the
' index of the first visible character
' (0 for the first character in the control).

Function GetFirstVisibleLine(tb As TextBox) As Long
    GetFirstVisibleLine = SendMessageByVal(tb.hwnd, EM_GETFIRSTVISIBLELINE, 0, 0)
End Function

' Return the number of the line that contains the specified character.
' Both line and character numbers are zero-based.

Function LineFromChar(tb As TextBox, ByVal charIndex As Long) As Long
    LineFromChar = SendMessageByVal(tb.hwnd, EM_LINEFROMCHAR, charIndex, 0)
End Function

' Return the character offset of the first character of a line.

Function LineIndex(tb As TextBox, ByVal lineNum As Long) As Long
    LineIndex = SendMessageByVal(tb.hwnd, EM_LINEINDEX, lineNum, 0)
End Function

' Return the length of the specified line.

Function LineLength(tb As TextBox, ByVal lineNum As Long) As Long
    Dim charOffset As Long
    ' Retrieve the character offset of the first character of the line.
    charOffset = SendMessageByVal(tb.hwnd, EM_LINEINDEX, lineNum, 0)
    ' Now it is possible to retrieve the length of the line.
    LineLength = SendMessageByVal(tb.hwnd, EM_LINELENGTH, charOffset, 0)
End Function

' Return the specified line.

Function GetLine(tb As TextBox, ByVal lineNum As Long) As String
    Dim charOffset As Long, lineLen As Long
    ' Retrieve the character offset of the first character of the line.
    charOffset = SendMessageByVal(tb.hwnd, EM_LINEINDEX, lineNum, 0)
    ' Now it is possible to retrieve the length of the line.
    lineLen = SendMessageByVal(tb.hwnd, EM_LINELENGTH, charOffset, 0)
    ' Extract the line text.
    GetLine = Mid$(tb.Text, charOffset + 1, lineLen)
End Function

' Get the line/column coordinates of a given character (both are zero-based).
' If charIndex is negative, it returns the coordinates of the caret

Sub GetLineColumn(tb As TextBox, ByVal charIndex As Long, line As Long, column As Long)
    If charIndex < 0 Then charIndex = tb.SelStart
    ' Get the line number.
    line = SendMessageByVal(tb.hwnd, EM_LINEFROMCHAR, charIndex, 0)
    ' Get the column number by subtracting the line's start
    ' index from the caret position
    column = tb.SelStart - SendMessageByVal(tb.hwnd, EM_LINEINDEX, line, 0)
End Sub

' Return an array with all the lines in the control.
' If the second optional argument is True, the trailing CR-LF is preserved.

Function GetAllLines(tb As TextBox, Optional KeepHardLineBreaks As Boolean) As String()
    Dim result() As String, i As Long
    ' Activate soft line breaks. A soft line break is marked by the
    ' CR-Cr-LF sequence.
    SendMessageByVal tb.hwnd, EM_FMTLINES, True, 0
    ' Retrieve all the lines in one operation. This operation will leave
    ' a trailing CR character for soft line breaks.
    result() = Split(tb.Text, vbCrLf)
    ' We need a loop to trim the trailing CR character. If the second
    ' argument is true, we need to manually add a CR-LF pair to all
    ' the lines that don't contain such trailing CR char.
    For i = 0 To UBound(result)
        If Right$(result(i), 1) = vbCr Then
            result(i) = Left$(result(i), Len(result(i)) - 1)
        ElseIf KeepHardLineBreaks Then
            result(i) = result(i) & vbCrLf
        End If
    Next
    ' Deactivate soft line breaks.
    SendMessageByVal tb.hwnd, EM_FMTLINES, False, 0
    GetAllLines = result()
End Function

' Get the formatting rectangle.

Sub GetRect(tb As TextBox, Left As Long, Top As Long, Right As Long, Bottom As Long)
    Dim lpRect As RECT
    SendMessage tb.hwnd, EM_GETRECT, 0, lpRect
    Left = lpRect.Left
    Top = lpRect.Top
    Right = lpRect.Right
    Bottom = lpRect.Bottom
End Sub

' Set the formatting rectangle and refresh the control.

Sub SetRect(tb As TextBox, ByVal Left As Long, ByVal Top As Long, ByVal Right As Long, ByVal Bottom As Long)
    Dim lpRect As RECT
    lpRect.Left = Left
    lpRect.Top = Top
    lpRect.Right = Right
    lpRect.Bottom = Bottom
    SendMessage tb.hwnd, EM_SETRECT, 0, lpRect
End Sub

' Scroll the contents of the control.
' Positive values scroll left and up, negative values
' scroll right and down.

Sub LineScroll(tb As TextBox, ByVal HorizScroll As Long, ByVal VertScroll As Long)
    SendMessageByVal tb.hwnd, EM_LINESCROLL, HorizScroll, VertScroll
End Sub

' Ensures that the caret is visible.

Sub ScrollCaret(tb As TextBox)
    SendMessageByVal tb.hwnd, EM_SCROLLCARET, 0, 0
End Sub

' Set tab stops. Each element of the array is expressed in dialog units,
' where each dialog unit is 1/4 of the average character width.

Sub SetTabStops(tb As TextBox, tabStops() As Long)
    Dim numEls As Long
    numEls = UBound(tabStops) - LBound(tabStops) + 1
    SendMessage tb.hwnd, EM_SETTABSTOPS, numEls, tabStops(LBound(tabStops))
End Sub

' Set the tab stop distance, expressed in dialog units.

Sub SetTabStopDistance(tb As TextBox, ByVal distance As Long)
    SendMessage tb.hwnd, EM_SETTABSTOPS, 1, distance
End Sub
